/*****************************************************************************

	File		: HTBlinds.c
	Last modified	: 04.07.95	

	Author		: Markus Marti, marmarti@iiic.ethz.ch

******************************************************************************
	Description	: This program handles the Blinds stream class
			  to give documents back to the user in a form
			  that is suitable for visually-impaired or
			  blind preople.

******************************************************************************
	includes
*****************************************************************************/
#include <time.h>

#include "HTBlinds.h"
#include "reader.h"

#include "HTSUtils.h"
#include "HTDaemon.h"

/*****************************************************************************
	constants, variables & datatypes
*****************************************************************************/

#define Blinds_BUF_SIZE 8096	/* buffer-size */

typedef enum {			/* defines states which are needed to */
    Blinds_EOL = 0,		/* decide how the stream has to be handled */
    Blinds_LINE,
    Blinds_BODY,
    Blinds_JUNK
} Blinds_State;

struct _HTStream {
    HTStreamClass *	isa;
    HTStream *		sink;
    Blinds_State	state;
    char		buffer[Blinds_BUF_SIZE];
    int			index;
    char *		content_type;
};
/****************************************************************************/

PRIVATE void Blinds_decide(HTStream *me)
/*	on entry :	me	input-stream to handle
	on exit  :	void
	description :	sets the next state
*/
{
    me->state = Blinds_BODY;
    (*me->sink->isa->put_block)(me->sink, me->buffer, me->index);
}


PRIVATE void Blinds_put_char(HTStream *me, char c)
/*	on entry :	me	input-stream to handle
			c	char to put out
	on exit  :	void
	description :	puts char c out
*/
{
    if (me->state == Blinds_JUNK)
	return;
    if (me->state == Blinds_BODY) {
	
	(*me->sink->isa->put_character)(me->sink,c);
	return;
    }
    me->buffer[me->index++] = c;
    if (c==LF) {
	if (me->state == Blinds_EOL) {
	    me->buffer[me->index] = 0;
	    Blinds_decide(me);
	}
	else me->state = Blinds_EOL;
    }
    else if (c!=CR)
	me->state = Blinds_LINE;
}


PRIVATE void Blinds_put_block(HTStream *me, CONST char *b, int l)
/*	on entry :	me	input-stream to handle
			b	string to put out
			l	length of string b
	on exit  :	void
	description :	puts string b out
*/
{
    if (!b || me->state==Blinds_JUNK)
	return;

    if (me->state != Blinds_BODY) {
	while (me->state!=Blinds_BODY && me->state!=Blinds_JUNK && l-- > 0) {
	    Blinds_put_char(me,*b);
	    b++;
	}
    }
    if (me->state == Blinds_BODY) {
	Blinds_send(me, b);	/* sending string to 'reader' */
    }
}


void Blinds_receive(HTStream *me, CONST char *b, int l)
/*	on entry :	me	input-stream
			b	string received from 'transformer'
			l	length of received string b
	on exit  :	void
	description :	receives the transformed string b from 'transformer'
			and sends it to Blinds_put_block
*/
{
    (*me->sink->isa->put_block)(me->sink,b,l);
}


PRIVATE void Blinds_put_string(HTStream *me, CONST char *s)
/*	on entry :	me	input-stream to handle
			s	string to put out
	on exit  :	void
	description :	puts string s out	
*/
{
    int len = s ? strlen(s) : 0;
    if (len > 0)
	Blinds_put_block(me,s,len);
}


PRIVATE void Blinds_free(HTStream *me)
/*	on entry :	me	input-stream to handle
	on exit  :	void
	description :	deallocates memory used by me	
*/
{
    Blinds_sendEOF(me);		/* send EOF-message to 'reader' */
    (*me->sink->isa->_free)(me->sink);
    if (me->state == Blinds_JUNK) {
	/* make sure that the log shows zero bytes transferred */
	out.content_length = 0;
    }
    free(me);	/* deallocate memory */
}


PRIVATE void Blinds_abort(HTStream *me, HTError e)
/*	on entry :	me	input-stream
			e	error-message
	on exit  :	void
	description :	is called in case of an error; deallocates memory
			used by me	
*/
{
    (*me->sink->isa->abort)(me->sink,e);
    free(me);
}


/*****************************************************************************
	Blinds stream class
*****************************************************************************/

PRIVATE HTStreamClass HTBlindsStreamClass =
/*	description :	defines the HTBlindsStreamClass
*/
{
    "BlindsStream",
    Blinds_free,
    Blinds_abort,
    Blinds_put_char,
    Blinds_put_string,
    Blinds_put_block
};


PUBLIC HTStream * HTBlindsStream(HTStream *sink)
/*	on entry :	me	input-stream to handle
	on exit  :	void
*/
{
    HTStream * me = (HTStream*)calloc(1,sizeof(HTStream));
    if (!me) outofmem(__FILE__, "HTBlindsStream");

    me->isa = &HTBlindsStreamClass;
    me->sink = sink;
    Reset();		/* resets the reader */
    init_transform();	/* initialize transformer */

    return me;
}

